@@ -43,10 +43,6 @@
   #include <X11/XF86keysym.h>
 #endif /* HAVE_MMKEYS */
 
-#ifdef HILDON
-  #include <hildon/hildon-program.h>
-#endif
-
 #include "cheese-countdown.h"
 #include "cheese-effect-chooser.h"
 #include "cheese-fileutil.h"
@@ -114,6 +110,11 @@ typedef struct
   GtkWidget *main_vbox;
   GtkWidget *video_vbox;
 
+  GtkWidget *netbook_alignment;
+  GtkWidget *toolbar_alignment;
+  GtkWidget *effect_button_alignment;
+  GtkWidget *togglegroup_alignment;
+
   GtkWidget *effect_frame;
   GtkWidget *effect_alignment;
   GtkWidget *effect_chooser;
@@ -155,11 +156,6 @@ typedef struct
   GtkWidget *take_picture;
   GtkWidget *take_picture_fullscreen;
 
-#ifdef HILDON
-  GtkWidget *main_hbox;
-  GtkWidget *subwindow;
-#endif
-
   GtkActionGroup *actions_account_photo;
   GtkActionGroup *actions_countdown;
   GtkActionGroup *actions_effects;
@@ -175,6 +171,7 @@ typedef struct
   GtkActionGroup *actions_video;
   GtkActionGroup *actions_burst;
   GtkActionGroup *actions_fullscreen;
+  GtkActionGroup *actions_wide_mode;
 
   GtkUIManager *ui_manager;
 
@@ -183,6 +180,7 @@ typedef struct
   int audio_play_counter;
 
   gint repeat_count;
+  gboolean is_bursting;
 
   CheeseFlash *flash;
 } CheeseWindow;
@@ -325,6 +323,68 @@ cheese_window_fullscreen_motion_notify_cb (GtkWidget      *widget,
   return FALSE;
 }
 
+static void
+cheese_window_toggle_wide_mode (GtkWidget *widget, CheeseWindow *cheese_window)
+{
+  gboolean toggled = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (widget));
+
+  gtk_widget_set_size_request (cheese_window->notebook,
+                               GTK_WIDGET (cheese_window->notebook)->allocation.width,
+                               GTK_WIDGET (cheese_window->notebook)->allocation.height);
+
+  /* set a single column in wide mode */
+  gtk_icon_view_set_columns (GTK_ICON_VIEW (cheese_window->thumb_view), toggled ? 1 : G_MAXINT);
+  /* switch thumb_nav mode */
+  eog_thumb_nav_set_vertical (EOG_THUMB_NAV (cheese_window->thumb_nav), toggled);
+
+  /* reparent thumb_view */
+  g_object_ref (cheese_window->thumb_scrollwindow);
+  if (toggled) {
+    gtk_container_remove (GTK_CONTAINER (cheese_window->video_vbox), cheese_window->thumb_scrollwindow);
+    gtk_container_add (GTK_CONTAINER (cheese_window->netbook_alignment), cheese_window->thumb_scrollwindow);
+    g_object_unref (cheese_window->thumb_scrollwindow);
+  } else {
+    gtk_container_remove (GTK_CONTAINER (cheese_window->netbook_alignment), cheese_window->thumb_scrollwindow);
+    gtk_box_pack_end (GTK_BOX (cheese_window->video_vbox), cheese_window->thumb_scrollwindow, FALSE, FALSE, 0);
+    g_object_unref (cheese_window->thumb_scrollwindow);
+    eog_thumb_nav_set_policy (EOG_THUMB_NAV (cheese_window->thumb_nav),
+                              GTK_POLICY_AUTOMATIC,
+                              GTK_POLICY_NEVER);
+  }
+
+  /* update spacing */
+  /* NOTE: be really carefull when changing the ui file to update spacing
+   * values here too! */
+  if (toggled) {
+    g_object_set (G_OBJECT (cheese_window->toolbar_alignment),
+                  "bottom-padding", 10, NULL);
+    g_object_set (G_OBJECT (cheese_window->togglegroup_alignment),
+                  "left-padding", 6, NULL);
+    g_object_set (G_OBJECT (cheese_window->effect_button_alignment),
+                  "right-padding", 0, NULL);
+    g_object_set (G_OBJECT (cheese_window->netbook_alignment),
+                  "left-padding", 6, NULL);
+  } else {
+    g_object_set (G_OBJECT (cheese_window->toolbar_alignment),
+                  "bottom-padding", 6, NULL);
+    g_object_set (G_OBJECT (cheese_window->togglegroup_alignment),
+                  "left-padding", 24, NULL);
+    g_object_set (G_OBJECT (cheese_window->effect_button_alignment),
+                  "right-padding", 24, NULL);
+    g_object_set (G_OBJECT (cheese_window->netbook_alignment),
+                  "left-padding", 0, NULL);
+  }
+
+  gtk_container_resize_children (GTK_CONTAINER (cheese_window->thumb_scrollwindow));
+
+  GtkRequisition req;
+  gtk_widget_size_request (cheese_window->window, &req);
+  gtk_window_resize (GTK_WINDOW (cheese_window->window), req.width, req.height);
+  gtk_widget_set_size_request (cheese_window->notebook, -1, -1);
+
+  g_object_set (cheese_window->gconf, "gconf_prop_wide_mode", toggled, NULL);
+}
+
 static void
 cheese_window_toggle_fullscreen (GtkWidget *widget, CheeseWindow *cheese_window)
 {
@@ -402,8 +462,12 @@ static void
 cheese_window_photo_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
   gdk_threads_enter ();
-  gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
-  gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, TRUE);
+  if (!cheese_window->is_bursting) {
+    gtk_action_group_set_sensitive (cheese_window->actions_effects, TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_toggle, TRUE);
+    gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
+    gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, TRUE);
+  }
   gdk_flush ();
   gdk_threads_leave ();
 }
@@ -1293,7 +1357,7 @@ cheese_window_escape_key_cb (CheeseWindow *cheese_window,
 }
 
 static gboolean
-cheese_window_take_burst_photo (gpointer data)
+cheese_window_take_photo (gpointer data)
 {
   gboolean      countdown;
   CheeseWindow *cheese_window = (CheeseWindow *) data;
@@ -1333,33 +1397,11 @@ cheese_window_take_burst_photo (gpointer data)
   gtk_widget_set_sensitive (cheese_window->take_picture, FALSE);
   gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, FALSE);
 
-  cheese_window->repeat_count--;
-  if (cheese_window->repeat_count <= 0)
-  {
-    return FALSE;
-  }
-  return TRUE;
-}
-
-static void
-cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window)
-{
-  char *str;
-
-  if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
-  {
-    cheese_window->repeat_count = 1;
-    cheese_window_take_burst_photo (cheese_window);
-
-    /* FIXME: set menu inactive */
-  }
-  else if (cheese_window->webcam_mode == WEBCAM_MODE_BURST)
-  {
-    guint    repeat_delay = 1000;
-    gboolean countdown;
+  if (cheese_window->webcam_mode == WEBCAM_MODE_BURST) {
+    guint repeat_delay = 1000;
+    gboolean countdown = FALSE;
 
     g_object_get (cheese_window->gconf, "gconf_prop_burst_delay", &repeat_delay, NULL);
-    g_object_get (cheese_window->gconf, "gconf_prop_burst_repeat", &cheese_window->repeat_count, NULL);
     g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
 
     if (countdown && repeat_delay < 5000)
@@ -1368,11 +1410,36 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
        * Magic number chosen via expiriment on Netbook */
       repeat_delay = 5000;
     }
-
-    g_timeout_add (repeat_delay, &cheese_window_take_burst_photo, cheese_window);
+    /* start burst mode phot series */
+    if (!cheese_window->is_bursting) {
+      g_timeout_add (repeat_delay, cheese_window_take_photo, cheese_window);
+      cheese_window->is_bursting = TRUE;
+    }
+    cheese_window->repeat_count--;
+    if (cheese_window->repeat_count > 0)
+    {
+      return TRUE;
+    }
   }
-  else if (cheese_window->webcam_mode == WEBCAM_MODE_VIDEO)
-  {
+  cheese_window->is_bursting = FALSE;
+
+  return FALSE;
+}
+
+static void
+cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window)
+{
+  char     *str;
+
+  switch (cheese_window->webcam_mode) {
+  case WEBCAM_MODE_BURST:
+    gtk_action_group_set_sensitive (cheese_window->actions_effects, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_toggle, FALSE);
+    g_object_get (cheese_window->gconf, "gconf_prop_burst_repeat", &cheese_window->repeat_count, NULL); /* reset burst counter */
+  case WEBCAM_MODE_PHOTO:
+    cheese_window_take_photo (cheese_window);
+    break;
+  case WEBCAM_MODE_VIDEO:
     if (!cheese_window->recording)
     {
       gtk_action_group_set_sensitive (cheese_window->actions_effects, FALSE);
@@ -1397,6 +1464,10 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
     {
       cheese_window_stop_recording (cheese_window);
     }
+    break;
+  default:
+    g_assert_not_reached ();
+    break;
   }
 }
 
@@ -1438,6 +1509,10 @@ static const GtkToggleActionEntry action_entries_fullscreen[] = {
   {"Fullscreen", GTK_STOCK_FULLSCREEN, NULL, "F11", NULL, G_CALLBACK (cheese_window_toggle_fullscreen), FALSE},
 };
 
+static const GtkToggleActionEntry action_entries_wide_mode[] = {
+  {"WideMode", NULL, N_("_Wide mode"), NULL, NULL, G_CALLBACK (cheese_window_toggle_wide_mode), FALSE},
+};
+
 static const GtkRadioActionEntry action_entries_toggle[] = {
   {"Photo", NULL, N_("_Photo"), NULL, NULL, 0},
   {"Video", NULL, N_("_Video"), NULL, NULL, 1},
@@ -1606,15 +1681,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   GError     *error = NULL;
   char       *path;
   GtkBuilder *builder;
-
-#ifdef HILDON
-  HildonProgram *program = hildon_program_get_instance ();
-  GtkWidget     *menu;
-  GtkWidget     *menuitem;
-#else
   GtkWidget *menubar;
-#endif
-
 
   cheese_window->info_bar = NULL;
 
@@ -1638,6 +1705,10 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->label_take_photo            = GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo"));
   cheese_window->label_video                 = GTK_WIDGET (gtk_builder_get_object (builder, "label_video"));
   cheese_window->main_vbox                   = GTK_WIDGET (gtk_builder_get_object (builder, "main_vbox"));
+  cheese_window->netbook_alignment           = GTK_WIDGET (gtk_builder_get_object (builder, "netbook_alignment"));
+  cheese_window->togglegroup_alignment       = GTK_WIDGET (gtk_builder_get_object (builder, "togglegroup_alignment"));
+  cheese_window->effect_button_alignment     = GTK_WIDGET (gtk_builder_get_object (builder, "effect_button_alignment"));
+  cheese_window->toolbar_alignment           = GTK_WIDGET (gtk_builder_get_object (builder, "toolbar_alignment"));
   cheese_window->video_vbox                  = GTK_WIDGET (gtk_builder_get_object (builder, "video_vbox"));
   cheese_window->notebook                    = GTK_WIDGET (gtk_builder_get_object (builder, "notebook"));
   cheese_window->notebook_bar                = GTK_WIDGET (gtk_builder_get_object (builder, "notebook_bar"));
@@ -1666,6 +1737,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
     GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame_fullscreen"));
   cheese_window->button_exit_fullscreen = GTK_WIDGET (gtk_builder_get_object (builder, "button_exit_fullscreen"));
 
+  g_object_unref (builder);
+
   /* configure the popup position and size */
   GdkScreen *screen = gtk_window_get_screen (GTK_WINDOW (cheese_window->fullscreen_popup));
   gtk_window_set_default_size (GTK_WINDOW (cheese_window->fullscreen_popup),
@@ -1682,30 +1755,6 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                     G_CALLBACK (cheese_window_exit_fullscreen_button_clicked_cb),
                     cheese_window);
 
-#ifdef HILDON
-  /* Reparent widgets in case we use hildon. This saves us maintaining two
-   * GtkBuilder ui files
-   */
-  cheese_window->window    = hildon_window_new ();
-  cheese_window->main_hbox = gtk_hbox_new (FALSE, 0);
-  hildon_program_add_window (program, HILDON_WINDOW (cheese_window->window));
-  gtk_container_add (GTK_CONTAINER (cheese_window->window), cheese_window->main_hbox);
-  g_object_ref (cheese_window->thumb_scrollwindow);
-  g_object_ref (cheese_window->video_vbox);
-  gtk_container_remove (GTK_CONTAINER (cheese_window->video_vbox), cheese_window->thumb_scrollwindow);
-  gtk_container_remove (GTK_CONTAINER (cheese_window->main_vbox), cheese_window->video_vbox);
-  gtk_box_pack_start (GTK_BOX (cheese_window->main_hbox), cheese_window->video_vbox, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (cheese_window->main_hbox), GTK_WIDGET (cheese_window->thumb_scrollwindow),
-                      FALSE, FALSE, 0);
-  gtk_widget_destroy (cheese_window->main_vbox);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (cheese_window->thumb_scrollwindow),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  g_object_unref (cheese_window->thumb_scrollwindow);
-  g_object_unref (cheese_window->video_vbox);
-#endif
-
-  g_object_unref (builder);
-
   char *str = g_strconcat ("<b>", _("_Take a photo"), "</b>", NULL);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo_fullscreen), str);
@@ -1716,7 +1765,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture_fullscreen), FALSE);
 
   cheese_window->thumb_view = cheese_thumb_view_new ();
-  cheese_window->thumb_nav  = eog_thumb_nav_new (cheese_window->thumb_view, TRUE);
+  cheese_window->thumb_nav  = eog_thumb_nav_new (cheese_window->thumb_view, FALSE);
+
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_nav);
 
   /* show the scroll window to get it included in the size requisition done later */
@@ -1775,6 +1825,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                              action_entries_fullscreen,
                                                                              G_N_ELEMENTS (action_entries_fullscreen));
 
+  cheese_window->actions_wide_mode = cheese_window_toggle_action_group_new (cheese_window,
+                                                                            "ActionsWideMode",
+                                                                            action_entries_wide_mode,
+                                                                            G_N_ELEMENTS (action_entries_fullscreen));
+
   cheese_window->actions_preferences = cheese_window_action_group_new (cheese_window,
                                                                        "ActionsPreferences",
                                                                        action_entries_preferences,
@@ -1868,25 +1923,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   g_object_get (cheese_window->gconf, "gconf_prop_enable_delete", &enable_delete, NULL);
   gtk_action_set_visible (GTK_ACTION (action), enable_delete);
 
-#ifdef HILDON
-  menu     = gtk_menu_new ();
-  menuitem = gtk_menu_item_new_with_label (_("Quit"));
-  g_signal_connect (menuitem, "activate",
-                    G_CALLBACK (cheese_window_cmd_close),
-                    cheese_window);
-  gtk_menu_shell_append (menu, menuitem);
-
-  menuitem = gtk_menu_item_new_with_label (_("About"));
-  g_signal_connect (menuitem, "activate",
-                    G_CALLBACK (cheese_window_cmd_about),
-                    cheese_window);
-  gtk_menu_shell_append (menu, menuitem);
-
-  hildon_window_set_menu (HILDON_WINDOW (cheese_window->window), GTK_MENU (menu));
-#else
   menubar = gtk_ui_manager_get_widget (cheese_window->ui_manager, "/MainMenu");
   gtk_box_pack_start (GTK_BOX (cheese_window->main_vbox), menubar, FALSE, FALSE, 0);
-#endif
 
   cheese_window->thumb_view_popup_menu = gtk_ui_manager_get_widget (cheese_window->ui_manager,
                                                                     "/ThumbnailPopup");
@@ -2039,9 +2077,10 @@ setup_camera (CheeseWindow *cheese_window)
 }
 
 void
-cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
+cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server, gboolean startup_in_wide_mode)
 {
   CheeseWindow *cheese_window;
+  gboolean startup_in_wide_mode_saved;
 
   cheese_window = g_new0 (CheeseWindow, 1);
 
@@ -2051,6 +2090,7 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window->flash               = cheese_flash_new ();
   cheese_window->audio_play_counter  = 0;
   cheese_window->isFullscreen        = FALSE;
+  cheese_window->is_bursting         = FALSE;
 
   cheese_window->server = dbus_server;
 
@@ -2069,14 +2109,27 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window->webcam_mode = WEBCAM_MODE_PHOTO;
   cheese_window->recording   = FALSE;
 
+  g_object_get (cheese_window->gconf,
+                "gconf_prop_wide_mode",
+                &startup_in_wide_mode_saved,
+                NULL);
+
+  startup_in_wide_mode = startup_in_wide_mode_saved ? TRUE : startup_in_wide_mode;
+
+  if (startup_in_wide_mode) {
+    GtkAction *action = gtk_ui_manager_get_action (cheese_window->ui_manager, "/MainMenu/Cheese/WideMode");
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+  }
+
   /* handy trick to set default size of the drawing area while not
    * limiting its minimum size, thanks Owen! */
+
   GtkRequisition req;
-  gtk_widget_set_size_request (cheese_window->screen,
+  gtk_widget_set_size_request (cheese_window->notebook,
                                DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
   gtk_widget_size_request (cheese_window->window, &req);
-  gtk_window_set_default_size (GTK_WINDOW (cheese_window->window), req.width, req.height);
-  gtk_widget_set_size_request (cheese_window->screen, -1, -1);
+  gtk_window_resize (GTK_WINDOW (cheese_window->window), req.width, req.height);
+  gtk_widget_set_size_request (cheese_window->notebook, -1, -1);
 
   gtk_widget_show_all (cheese_window->window);
 
