@@ -23,13 +23,11 @@
   #include <cheese-config.h>
 #endif
 
-#include <stdio.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <gst/gst.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include "cheese-fileutil.h"
 #include "cheese-window.h"
@@ -38,7 +36,9 @@
 struct _CheeseOptions
 {
   gboolean verbose;
+  gboolean wide_mode;
   char *hal_device_id;
+  gboolean version;
 } CheeseOptions;
 
 void
@@ -50,6 +50,9 @@ cheese_print_handler (char *string)
 
   CheeseFileUtil *fileutil = cheese_fileutil_new ();
 
+  if (CheeseOptions.verbose)
+    fprintf (stdout, "%s", string);
+
   if (fp == NULL)
   {
     path = cheese_fileutil_get_log_path (fileutil);
@@ -80,9 +83,6 @@ cheese_print_handler (char *string)
 
   if (fp)
     fputs (string, fp);
-
-  if (CheeseOptions.verbose)
-    fprintf (stdout, "%s", string);
 }
 
 void
@@ -133,7 +133,7 @@ cheese_handle_files_from_before_224 (void)
       source_filename = g_build_filename (path_from_before_224, name, NULL);
       source          = g_file_new_for_path (source_filename);
       target          = g_file_new_for_path (target_filename);
-      printf ("copying %s to %s\n", source_filename, target_filename);
+      g_print ("copying %s to %s\n", source_filename, target_filename);
       g_file_move (source, target, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, NULL);
       g_free (source_filename);
       g_free (target_filename);
@@ -156,11 +156,17 @@ main (int argc, char **argv)
 {
   GOptionContext *context;
   CheeseDbus     *dbus_server;
-  GError *error = NULL;
+  GError         *error = NULL;
 
   GOptionEntry options[] = {
-    {"verbose",    'v', 0,                    G_OPTION_ARG_NONE,   &CheeseOptions.verbose,       _("Be verbose"), NULL},
-    {"hal-device", 'd', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &CheeseOptions.hal_device_id, NULL,            NULL},
+    {"verbose",    'v', 0, G_OPTION_ARG_NONE, &CheeseOptions.verbose,
+     _("Be verbose"), NULL},
+    {"wide",       'w', 0, G_OPTION_ARG_NONE, &CheeseOptions.wide_mode,
+     _("Enable wide mode"), NULL},
+    {"hal-device", 'd', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &CheeseOptions.hal_device_id,
+     NULL, NULL},
+    {"version", 0, 0, G_OPTION_ARG_NONE,   &CheeseOptions.version,
+     _("output version information and exit"), NULL},
     {NULL}
   };
 
@@ -181,7 +187,8 @@ main (int argc, char **argv)
   g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
   g_option_context_add_group (context, gtk_get_option_group (TRUE));
   g_option_context_add_group (context, gst_init_get_option_group ());
-  if (g_option_context_parse (context, &argc, &argv, &error) == FALSE) {
+  if (g_option_context_parse (context, &argc, &argv, &error) == FALSE)
+  {
     gchar *help_text = g_option_context_get_help (context, TRUE, NULL);
     g_print ("%s\n\n%s", error->message, help_text);
     g_free (help_text);
@@ -191,6 +198,12 @@ main (int argc, char **argv)
   }
   g_option_context_free (context);
 
+  if (CheeseOptions.version)
+  {
+    g_print ("Cheese " VERSION " \n");
+    return 0;
+  }
+
   dbus_server = cheese_dbus_new ();
   if (dbus_server == NULL)
   {
@@ -198,9 +211,6 @@ main (int argc, char **argv)
     return -1;
   }
 
-  /* Needed for gnome_thumbnail functions */
-  gnome_vfs_init ();
-
   g_set_print_handler ((GPrintFunc) cheese_print_handler);
   g_print ("Cheese " VERSION " \n");
 
@@ -209,7 +219,7 @@ main (int argc, char **argv)
                                      APPNAME_DATA_DIR G_DIR_SEPARATOR_S "icons");
 
   cheese_handle_files_from_before_224 ();
-  cheese_window_init (CheeseOptions.hal_device_id, dbus_server);
+  cheese_window_init (CheeseOptions.hal_device_id, dbus_server, CheeseOptions.wide_mode);
 
   gdk_threads_enter ();
   gtk_main ();
